@@ -7,6 +7,7 @@
 from __future__ import absolute_import, division, print_function
 
 import argparse
+
 from glob2 import glob
 import logging
 import os
@@ -204,8 +205,22 @@ different sets of packages."""
     p.add_argument(
         "--verify",
         action="store_true",
+        help="run verification on recipes or packages when building",
+        default=cc_conda_build.get('verify', 'true').lower() == 'true',
+    )
+    p.add_argument(
+        "--no-verify",
+        action="store_false",
+        dest="verify",
         help="do not run verification on recipes or packages when building",
-        default=cc_conda_build.get('verify', 'false').lower() == 'true',
+        default=cc_conda_build.get('verify', 'true').lower() == 'true',
+    )
+    p.add_argument(
+        "--strict-verify",
+        action="store_true",
+        dest="exit_on_verify_error",
+        help="Exit if any conda-verify check fail, instead of only printing them",
+        default=cc_conda_build.get('exit_on_verify_error', 'false').lower() == 'true',
     )
     p.add_argument(
         "--output-folder",
@@ -412,7 +427,7 @@ def execute(args):
     else:
         outputs = api.build(args.recipe, post=args.post, build_only=args.build_only,
                             notest=args.notest, already_built=None, config=config,
-                            verify=args.verify)
+                            verify=args.verify, variants=args.variants)
 
     if not args.output and len(utils.get_build_folders(config.croot)) > 0:
         build.print_build_intermediate_warning(config)
