@@ -15,7 +15,7 @@ from .conda_interface import hashsum_file
 
 from conda_build.os_utils import external
 from conda_build.conda_interface import url_path, CondaHTTPError
-from conda_build.utils import (tar_xf, unzip, safe_print_unicode, copy_into, on_win, ensure_list,
+from conda_build.utils import (decompressible_exts, tar_xf, safe_print_unicode, copy_into, on_win, ensure_list,
                                check_output_env, check_call_env, convert_path_for_cygwin_or_msys2,
                                get_logger, rm_rf, LoggingContext)
 
@@ -54,8 +54,8 @@ def download_to_cache(cache_folder, recipe_path, source_dict):
             break
     else:
         log = get_logger(__name__)
-        log.warn("No hash (md5, sha1, sha256) provided.  Source download forced.  "
-                 "Add hash to recipe to use source cache.")
+        log.warn("No hash (md5, sha1, sha256) provided for {}.  Source download forced.  "
+                 "Add hash to recipe to use source cache.".format(unhashed_fn))
     path = join(cache_folder, fn)
     if isfile(path):
         print('Found source in cache: %s' % fn)
@@ -139,21 +139,17 @@ def unpack(source_dict, src_dir, cache_folder, recipe_path, croot, verbose=False
         print("Extracting download")
     with TemporaryDirectory(dir=croot) as tmpdir:
         unhashed_dest = os.path.join(tmpdir, unhashed_fn)
-        if src_path.lower().endswith(('.tar.gz', '.tar.bz2', '.tgz', '.tar.xz',
-                '.tar', 'tar.z')):
+        if src_path.lower().endswith(decompressible_exts):
             tar_xf(src_path, tmpdir)
-        elif src_path.lower().endswith('.zip'):
-            unzip(src_path, tmpdir)
-        elif src_path.lower().endswith('.whl'):
-            # copy wheel itself *and* unpack it
-            # This allows test_files or about.license_file to locate files in the wheel,
-            # as well as `pip install name-version.whl` as install command
-            unzip(src_path, tmpdir)
-            copy_into(src_path, unhashed_dest, timeout, locking=locking)
         else:
             # In this case, the build script will need to deal with unpacking the source
             print("Warning: Unrecognized source format. Source file will be copied to the SRC_DIR")
             copy_into(src_path, unhashed_dest, timeout, locking=locking)
+        if src_path.lower().endswith('.whl'):
+            # copy wheel itself *and* unpack it
+            # This allows test_files or about.license_file to locate files in the wheel,
+            # as well as `pip install name-version.whl` as install command
+            copy_into(src_path, unhashed_dest, timeout, locking=locking)
         flist = os.listdir(tmpdir)
         folder = os.path.join(tmpdir, flist[0])
         if len(flist) == 1 and os.path.isdir(folder):
@@ -203,19 +199,29 @@ def git_mirror_checkout_recursive(git, mirror_dir, checkout_dir, git_url, git_ca
     if not isdir(os.path.dirname(mirror_dir)):
         os.makedirs(os.path.dirname(mirror_dir))
     if isdir(mirror_dir):
-        if git_ref != 'HEAD':
-            check_call_env([git, 'fetch'], cwd=mirror_dir, stdout=stdout, stderr=stderr)
-        else:
-            # Unlike 'git clone', fetch doesn't automatically update the cache's HEAD,
-            # So here we explicitly store the remote HEAD in the cache's local refs/heads,
-            # and then explicitly set the cache's HEAD.
-            # This is important when the git repo is a local path like "git_url: ../",
-            # but the user is working with a branch other than 'master' without
-            # explicitly providing git_rev.
-            check_call_env([git, 'fetch', 'origin', '+HEAD:_conda_cache_origin_head'],
-                       cwd=mirror_dir, stdout=stdout, stderr=stderr)
-            check_call_env([git, 'symbolic-ref', 'HEAD', 'refs/heads/_conda_cache_origin_head'],
-                       cwd=mirror_dir, stdout=stdout, stderr=stderr)
+        try:
+            if git_ref != 'HEAD':
+                check_call_env([git, 'fetch'], cwd=mirror_dir, stdout=stdout, stderr=stderr)
+            else:
+                # Unlike 'git clone', fetch doesn't automatically update the cache's HEAD,
+                # So here we explicitly store the remote HEAD in the cache's local refs/heads,
+                # and then explicitly set the cache's HEAD.
+                # This is important when the git repo is a local path like "git_url: ../",
+                # but the user is working with a branch other than 'master' without
+                # explicitly providing git_rev.
+                check_call_env([git, 'fetch', 'origin', '+HEAD:_conda_cache_origin_head'],
+                           cwd=mirror_dir, stdout=stdout, stderr=stderr)
+                check_call_env([git, 'symbolic-ref', 'HEAD', 'refs/heads/_conda_cache_origin_head'],
+                           cwd=mirror_dir, stdout=stdout, stderr=stderr)
+        except CalledProcessError:
+            msg = ("Failed to update local git cache. "
+                   "Deleting local cached repo: {} ".format(mirror_dir))
+            print(msg)
+
+            # Maybe the failure was caused by a corrupt mirror directory.
+            # Delete it so the user can try again.
+            shutil.rmtree(mirror_dir)
+            raise
     else:
         args = [git, 'clone', '--mirror']
         if git_depth > 0:
@@ -455,6 +461,7 @@ def get_repository_info(recipe_path):
             return "Origin {}, commit {}".format(origin, rev)
         elif isdir(join(recipe_path, ".svn")):
             info = check_output_env(["svn", "info"], cwd=recipe_path)
+            info = info.decode("utf-8")  # Py3 returns a byte string, but re needs unicode or str.
             server = re.search("Repository Root: (.*)$", info, flags=re.M).group(1)
             revision = re.search("Revision: (.*)$", info, flags=re.M).group(1)
             return "{}, Revision {}".format(server, revision)
@@ -516,14 +523,14 @@ def _guess_patch_strip_level(filesstr, src_dir):
 
 
 def _get_patch_file_details(path):
-    re_files = re.compile('^(?:---|\+\+\+) ([^\n\t]+)')
+    re_files = re.compile(r'^(?:---|\+\+\+) ([^\n\t]+)')
     files = set()
     with io.open(path, errors='ignore') as f:
         files = []
         first_line = True
         is_git_format = True
         for l in f.readlines():
-            if first_line and not re.match('From [0-9a-f]{40}', l):
+            if first_line and not re.match(r'From [0-9a-f]{40}', l):
                 is_git_format = False
             first_line = False
             m = re_files.search(l)
@@ -639,7 +646,8 @@ def provide(metadata):
                         verbose=metadata.config.verbose, timeout=metadata.config.timeout,
                         locking=metadata.config.locking)
             elif 'path' in source_dict:
-                path = normpath(abspath(join(metadata.path, source_dict['path'])))
+                source_path = os.path.expanduser(source_dict['path'])
+                path = normpath(abspath(join(metadata.path, source_path)))
                 if metadata.config.verbose:
                     print("Copying %s to %s" % (path, src_dir))
                 # careful here: we set test path to be outside of conda-build root in setup.cfg.
@@ -651,8 +659,10 @@ def provide(metadata):
                     os.makedirs(src_dir)
 
             patches = ensure_list(source_dict.get('patches', []))
+            recipe_dir = (metadata.path or
+                          metadata.meta.get('extra', {}).get('parent_recipe', {}).get('path', ''))
             for patch in patches:
-                apply_patch(src_dir, join(metadata.path, patch), metadata.config, git)
+                apply_patch(src_dir, join(recipe_dir, patch), metadata.config, git)
 
     except CalledProcessError:
         shutil.move(metadata.config.work_dir, metadata.config.work_dir + '_failed_provide')
