@@ -1,12 +1,12 @@
 from __future__ import absolute_import, division, print_function
 
-import base64
 from collections import defaultdict
 import contextlib
 import fnmatch
 import hashlib
 import json
 from locale import getpreferredencoding
+import libarchive
 import logging
 import logging.config
 import mmap
@@ -23,10 +23,15 @@ import tarfile
 import tempfile
 import time
 import yaml
-import zipfile
 
 import filelock
 
+try:
+    from conda.base.constants import CONDA_TARBALL_EXTENSIONS
+except Exception:
+    from conda.base.constants import CONDA_TARBALL_EXTENSION
+    CONDA_TARBALL_EXTENSIONS = (CONDA_TARBALL_EXTENSION,)
+
 from .conda_interface import hashsum_file, md5_file, unix_path_to_win, win_path_to_unix
 from .conda_interface import PY3, iteritems
 from .conda_interface import root_dir, pkgs_dirs
@@ -39,6 +44,7 @@ from .conda_interface import conda_43, Dist
 from .conda_interface import context
 # NOQA because it is not used in this file.
 from conda_build.conda_interface import rm_rf as _rm_rf # NOQA
+from conda_build.exceptions import BuildLockError
 from conda_build.os_utils import external
 
 if PY3:
@@ -53,6 +59,7 @@ if PY3:
     # NOQA because it is not used in this file.
     from contextlib import ExitStack  # NOQA
     PermissionError = PermissionError  # NOQA
+    FileNotFoundError = FileNotFoundError
 else:
     from glob2 import glob as glob2_glob
 
@@ -64,6 +71,7 @@ else:
     # NOQA because it is not used in this file.
     from contextlib2 import ExitStack  # NOQA
     PermissionError = OSError
+    FileNotFoundError = OSError
 
 on_win = (sys.platform == 'win32')
 
@@ -344,17 +352,13 @@ def get_recipe_abspath(recipe):
     if not PY3:
         recipe = recipe.decode(getpreferredencoding() or 'utf-8')
     if isfile(recipe):
-        if recipe.endswith(('.tar', '.tar.gz', '.tgz', '.tar.bz2')):
+        if recipe.lower().endswith(decompressible_exts) or recipe.lower().endswith(CONDA_TARBALL_EXTENSIONS):
             recipe_dir = tempfile.mkdtemp()
-            t = tarfile.open(recipe, 'r:*')
-            t.extractall(path=recipe_dir)
+            tar_xf(recipe, recipe_dir)
             # At some stage the old build system started to tar up recipes.
             recipe_tarfile = os.path.join(recipe_dir, 'info', 'recipe.tar')
             if isfile(recipe_tarfile):
-                t2 = tarfile.open(recipe_tarfile, 'r:*')
-                t2.extractall(path=os.path.join(recipe_dir, 'info'))
-                t2.close()
-            t.close()
+                tar_xf(recipe_tarfile, os.path.join(recipe_dir, 'info'))
             need_cleanup = True
         else:
             print("Ignoring non-recipe: %s" % recipe)
@@ -369,23 +373,44 @@ def get_recipe_abspath(recipe):
 
 @contextlib.contextmanager
 def try_acquire_locks(locks, timeout):
-    """Try to acquire all locks.  If any lock can't be immediately acquired, free all locks
+    """Try to acquire all locks.
+
+    If any lock can't be immediately acquired, free all locks.
+    If the timeout is reached withou acquiring all locks, free all locks and raise.
 
     http://stackoverflow.com/questions/9814008/multiple-mutex-locking-strategies-and-why-libraries-dont-use-address-comparison
     """
     t = time.time()
     while (time.time() - t < timeout):
-        for lock in locks:
-            try:
+        # Continuously try to acquire all locks.
+        # By passing a short timeout to each individual lock, we give other
+        # processes that might be trying to acquire the same locks (and may
+        # already hold some of them) a chance to the remaining locks - and
+        # hopefully subsequently release them.
+        try:
+            for lock in locks:
                 lock.acquire(timeout=0.1)
-            except filelock.Timeout:
-                for lock in locks:
-                    lock.release()
-                break
-        break
-    yield
-    for lock in locks:
-        if lock:
+        except filelock.Timeout:
+            # If we failed to acquire a lock, it is important to release all
+            # locks we may have already acquired, to avoid wedging multiple
+            # processes that try to acquire the same set of locks.
+            # That is, we want to avoid a situation where processes 1 and 2 try
+            # to acquire locks A and B, and proc 1 holds lock A while proc 2
+            # holds lock B.
+            for lock in locks:
+                lock.release()
+        else:
+            break
+    else:
+        # If we reach this point, we weren't able to acquire all locks within
+        # the specified timeout. We shouldn't be holding any locks anymore at
+        # this point, so we just raise an exception.
+        raise BuildLockError('Failed to acquire all locks')
+
+    try:
+        yield
+    finally:
+        for lock in locks:
             lock.release()
 
 
@@ -565,7 +590,10 @@ def get_lock(folder, timeout=90):
     b_location = location
     if hasattr(b_location, 'encode'):
         b_location = b_location.encode()
-    lock_filename = base64.urlsafe_b64encode(b_location)[:20]
+
+    # Hash the entire filename to avoid collisions.
+    lock_filename = hashlib.sha256(b_location).hexdigest()
+
     if hasattr(lock_filename, 'decode'):
         lock_filename = lock_filename.decode()
     for locks_dir in _lock_folders:
@@ -617,60 +645,22 @@ def relative(f, d='lib'):
     return '/'.join(((['..'] * len(f)) if f else ['.']) + d)
 
 
-def tar_xf(tarball, dir_path, mode='r:*'):
-    if tarball.lower().endswith('.tar.z'):
-        uncompress = external.find_executable('uncompress')
-        if not uncompress:
-            uncompress = external.find_executable('gunzip')
-        if not uncompress:
-            sys.exit("""\
-uncompress (or gunzip) is required to unarchive .z source files.
-""")
-        check_call_env([uncompress, '-f', tarball])
-        tarball = tarball[:-2]
-    if not PY3 and tarball.endswith('.tar.xz'):
-        unxz = external.find_executable('unxz')
-        if not unxz:
-            sys.exit("""\
-unxz is required to unarchive .xz source files.
-""")
-
-        check_call_env([unxz, '-f', '-k', tarball])
-        tarball = tarball[:-3]
-    t = tarfile.open(tarball, mode)
-    members = t.getmembers()
-    for i, member in enumerate(members, 0):
-        if os.path.isabs(member.name):
-            member.name = os.path.relpath(member.name, '/')
-        if not os.path.realpath(member.name).startswith(os.getcwd()):
-            member.name = member.name.replace("../", "")
-        if not os.path.realpath(member.name).startswith(os.getcwd()):
-            sys.exit("tarball contains unsafe path: " + member.name)
-        members[i] = member
-
-    if not PY3:
-        t.extractall(path=dir_path.encode(codec))
-    else:
-        t.extractall(path=dir_path)
-    t.close()
+# This is the lowest common denominator of the formats supported by our libarchive/python-libarchive-c
+# packages across all platforms
+decompressible_exts = ('.7z', '.tar', '.tar.bz2', '.tar.gz', '.tar.lzma', '.tar.xz',
+                       '.tar.z', '.tgz', '.whl', '.zip', '.rpm', '.deb')
 
 
-def unzip(zip_path, dir_path):
-    z = zipfile.ZipFile(zip_path)
-    for info in z.infolist():
-        name = info.filename
-        if name.endswith('/'):
-            continue
-        path = join(dir_path, *name.split('/'))
-        dp = dirname(path)
-        if not isdir(dp):
-            os.makedirs(dp)
-        with open(path, 'wb') as fo:
-            fo.write(z.read(name))
-        unix_attributes = info.external_attr >> 16
-        if unix_attributes:
-            os.chmod(path, unix_attributes)
-    z.close()
+def tar_xf(tarball, dir_path):
+    flags = libarchive.extract.EXTRACT_TIME | \
+            libarchive.extract.EXTRACT_PERM | \
+            libarchive.extract.EXTRACT_SECURE_NODOTDOT | \
+            libarchive.extract.EXTRACT_SECURE_SYMLINKS | \
+            libarchive.extract.EXTRACT_SECURE_NOABSOLUTEPATHS
+    if not os.path.isabs(tarball):
+        tarball = os.path.join(os.getcwd(), tarball)
+    with tmp_chdir(dir_path):
+        libarchive.extract_file(tarball, flags)
 
 
 def file_info(path):
@@ -1042,21 +1032,26 @@ def find_recipe(path):
 
 
 class LoggingContext(object):
-    loggers = ['conda', 'binstar', 'install', 'conda.install', 'fetch', 'conda.instructions',
-               'fetch.progress', 'print', 'progress', 'dotupdate', 'stdoutlog', 'requests',
-               'conda.core.package_cache', 'conda.plan', 'conda.gateways.disk.delete',
-               'conda_build', 'conda_build.index']
+    default_loggers = ['conda', 'binstar', 'install', 'conda.install', 'fetch', 'conda.instructions',
+                       'fetch.progress', 'print', 'progress', 'dotupdate', 'stdoutlog', 'requests',
+                       'conda.core.package_cache', 'conda.plan', 'conda.gateways.disk.delete',
+                       'conda_build', 'conda_build.index']
 
-    def __init__(self, level=logging.WARN, handler=None, close=True):
+    def __init__(self, level=logging.WARN, handler=None, close=True, loggers=None):
         self.level = level
         self.old_levels = {}
         self.handler = handler
         self.close = close
         self.quiet = context.quiet
+        if not loggers:
+            self.loggers = LoggingContext.default_loggers
+        else:
+            self.loggers = loggers
 
     def __enter__(self):
-        for logger in LoggingContext.loggers:
-            log = logging.getLogger(logger)
+        for logger in self.loggers:
+            if isinstance(logger, string_types):
+                log = logging.getLogger(logger)
             self.old_levels[logger] = log.level
             log.setLevel(self.level if ('install' not in logger or
                                         self.level < logging.INFO) else self.level + 10)
@@ -1260,7 +1255,7 @@ def filter_info_files(files_list, prefix):
                     'info[\\\\/]run_exports.json',   # current
                     'info[\\\\/]git',
                     'info[\\\\/]recipe[\\\\/].*',
-                    'info[\\\\/]recipe_log.txt',
+                    'info[\\\\/]recipe_log.json',
                     'info[\\\\/]recipe.tar',
                     'info[\\\\/]test[\\\\/].*',
                     'info[\\\\/]LICENSE.txt',
@@ -1396,7 +1391,7 @@ def _equivalent(base_value, value, path):
     return equivalent
 
 
-def merge_or_update_dict(base, new, path, merge, raise_on_clobber=False):
+def merge_or_update_dict(base, new, path="", merge=True, raise_on_clobber=False):
     log = get_logger(__name__)
     for key, value in new.items():
         base_value = base.get(key, value)
@@ -1422,7 +1417,10 @@ def merge_or_update_dict(base, new, path, merge, raise_on_clobber=False):
                     raise_on_clobber):
                 log.debug('clobbering key {} (original value {}) with value {}'.format(key,
                                                                             base_value, value))
-            base[key] = value
+            if value is None:
+                del base[key]
+            else:
+                base[key] = value
     return base
 
 
@@ -1533,7 +1531,7 @@ spec_ver_needing_star_re = re.compile("^([0-9a-zA-Z\.]+)$")
 
 def ensure_valid_spec(spec, warn=False):
     if isinstance(spec, MatchSpec):
-        if (hasattr(spec, 'version') and spec.version and
+        if (hasattr(spec, 'version') and spec.version and (not hasattr(spec, 'build') or spec.build) and
                 spec_ver_needing_star_re.match(str(spec.version))):
             if str(spec.name) not in ('python', 'numpy') or str(spec.version) != 'x.x':
                 spec = MatchSpec("{} {}".format(str(spec.name), str(spec.version) + '.*'))
