@@ -17,6 +17,7 @@ from conda_build.conda_interface import string_types
 from conda_build.conda_interface import subdir
 from conda_build.conda_interface import cc_conda_build
 from conda_build.conda_interface import memoized
+from conda_build.utils import on_win
 
 DEFAULT_VARIANTS = {
     'python': '{0}.{1}'.format(sys.version_info.major, sys.version_info.minor),
@@ -25,7 +26,7 @@ DEFAULT_VARIANTS = {
     #    version to say what's in their standard library.
     'perl': '5.26.0',
     'lua': '5',
-    'r_base': '3.5',
+    'r_base': '3.4' if on_win else '3.5',
     'cpu_optimization_target': 'nocona',
     'pin_run_as_build': OrderedDict(python=OrderedDict(min_pin='x.x', max_pin='x.x')),
     'ignore_version': [],
@@ -234,6 +235,7 @@ def _combine_spec_dictionaries(specs, extend_keys=None, filter_keys=None, zip_ke
                             else:
                                 if k in values and any(subvalue not in values[k]
                                                     for subvalue in ensure_list(v)):
+
                                     raise ValueError("variant config in {} is ambiguous because it "
                                         "does not fully implement all zipped keys, or specifies "
                                         "a subspace that is not fully implemented.".format(
@@ -527,8 +529,8 @@ def get_vars(variants, loop_only=False):
     to the matrix dimensionality"""
     special_keys = ('pin_run_as_build', 'zip_keys', 'ignore_version')
     loop_vars = [k for k in variants[0] if k not in special_keys and
-                 (not loop_only or
-                  any(variant[k] != variants[0][k] for variant in variants[1:]))]
+                (not loop_only or
+                any(variant[k] != variants[0][k] for variant in variants[1:]))]
     return loop_vars
 
 
